/* styles for search components */

.ab-search-input {
  margin-bottom: 2em;
}

.ab-search-results {
}

.ab-search-result {
  padding-bottom: 0.6em;

  .title {
    font-size: 1.25em;
    font-weight: bold;
  }
}

.advanced-search-filters {
  .criterion {
    /* close button */
    > div:first-child .close {
      float: left;
    }
  }
}
